# Copyright 2008 Krystof Hoder.
#
# This file is part of SInE.
#
# SInE is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# SInE is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with SInE.  If not, see <http://www.gnu.org/licenses/>.


import os.path
import subprocess
import suspender
import re

def rel_to_abs(path):
	cur=os.path.dirname(__file__)
	return os.path.join(cur, path)	


class Engine(object):
	def get_cmd_line(self, **kwargs):
		pass
	def run(self, **kwargs):
		cmdl=self.get_cmd_line(**kwargs)
		subprocess.call(cmdl)
	def run_into_file(self, output_file, **kwargs):
		cmdl=self.get_cmd_line(**kwargs)
		
		#print "-running prover on %s with time limit %d" % (kwargs['problem_file'],kwargs['time'])
		out = open(output_file, 'w')
		try:
			subprocess.call(cmdl, stdout=out)
		finally:
			out.close()

	def get_process(self, output_file, **kwargs):
		cmdl=self.get_cmd_line(**kwargs)
		return suspender.Process(cmdl, output_file)
	def update_proof_axioms(self, output_file, axiom_set_set):
		axs=axiom_set_set.get_axioms()
		names=set(get_matches_groups(output_file, 'file\([^,]*, ([^)]*)\)'))
		for name in names:
			ax=axs.get(name, None)
			if ax:
				axiom_set_set.proof_axioms.add(ax)
	def proof_complete(self,output_file):
		pass
		
	@classmethod
	def get(cls, name):
		if name=='vampire':
			return Vampire()
		elif name=='eprover':
			return EProver()
                elif name=='leancop':
			return LeanCoP()

class Vampire(Engine):
	executable=rel_to_abs('./JUSource/jumpirefix')
	def get_cmd_line(self, problem_file, time):
		return [self.executable, '-t', str(int(time)), '--output_syntax', 'tptp', problem_file]
	def proof_complete(self,output_file):
		return len(get_matches_groups(output_file, ' PROVED ---'))>0
		
class EProver(Engine):
	executable=rel_to_abs('./e1.0temi/eproof.working')
	def get_cmd_line(self, problem_file, time):
		#return [self.executable, '-s', '-tLPO4', '-xAuto','-tAuto','--memory-limit=768', '--tptp3-format', '--cpu-limit=%s' % int(time), problem_file]
                return [self.executable, '-s', '-xAuto','-tAuto','--memory-limit=Auto', '--tptp3-format', '--cpu-limit=%s' % int(time), problem_file]
	def get_process(self, output_file, **kwargs):
		cmdl=self.get_cmd_line(**kwargs)
		return suspender.Process(cmdl, output_file, handle_subproc=True)
	
	def proof_complete(self,output_file):
		return len(get_matches_groups(output_file, '# SZS status Theorem'))>0
	

class LeanCoP(Engine):
	executable=rel_to_abs('./leancop_sine.sh')
	def get_cmd_line(self, problem_file, time):
		return [self.executable,  problem_file, str(int(time))]
	def proof_complete(self,output_file):
		return len(get_matches_groups(output_file, 'Theorem'))>0
	
	
def get_matches_groups(fname, pattern):
	exp=re.compile(pattern)

	res=[]	
	inp = open(fname)
	try:
		for line in inp:
			res.extend(exp.findall(line))
	finally:
		inp.close()
	return res
	
