#!/usr/bin/python

import sys
import re
import os
import os.path
import tempfile
import shutil
from optparse import OptionParser
from sets import Set

import engines
import tptp
import selection

temp_dir=tempfile.mkdtemp()
tptp.fname_dir=temp_dir

opt_parser=OptionParser()
opt_parser.add_option('-t', '--time', type='int', default=300)
opt_parser.add_option('-e', '--engine', type='string', default='vampire')
opt_parser.add_option('-p', '--axiom_prefix', type='string', default='')
opt_parser.add_option('-b', '--bypass', action="store_true")
opt_parser.add_option('-s', '--select_only', action="store_true")
opt_parser.add_option('-n', '--select_names', action="store_true")

opts, args=opt_parser.parse_args()

axiom_prefix=opts.axiom_prefix
if not axiom_prefix:
	if 'TPTP' in os.environ:
		axiom_prefix=os.environ['TPTP']
if axiom_prefix:
	tptp.AxiomSet.file_prefix=axiom_prefix

problem_fname=args[0]

problem=tptp.Problem(problem_fname)

if problem.axiom_sets:
	print 'Initializing axiom selector for axiom sets %s...' % reduce(lambda x,y:'%s, %s' % (x,y),problem.axiom_sets) 
	selector=selection.Selector(axiom_set_set=problem.axiom_sets)
	print 'Selecting axioms...'
	forms=selector.select_for_problem(problem)
else:
	forms=Set()
forms.update(problem.own_formulas.itervalues())

if opts.select_only:
	for form in forms:
		print form
	exit()
if opts.select_names:
	for form in forms:
		print form.name
	exit()

selected_fname=tptp.write_forms_to_file(problem, forms)
print 'Using %s formulas.' % len(forms)


engine=engines.Engine.get(opts.engine)

engine.run(problem_file=selected_fname, time=opts.time)

shutil.rmtree(temp_dir)