import os
import sys

from utils import props

import tptp
import suspender
import selection
from selection import SelectionParams

class BillingError(Exception):
	pass

def get_cpu_time():
	times=os.times()
	return reduce(lambda x,y:x+y, times[0:4])
def get_wall_time():
	return os.times()[4]

start_wall_time=get_wall_time()

class Timer(object):
	phases=[
		(0.10, SelectionParams()),
		(0.42, SelectionParams(select_proof_axioms=True)),
		(0.55, SelectionParams(select_proof_axioms=True, benevolence=1.2)),
		(0.60, SelectionParams(select_proof_axioms=True, benevolence=1.5)),
		(1, SelectionParams(select_proof_axioms=True, benevolence=2)),
		]

	def __init__(self, cpu_limit, problems, engine):
		self.__class__.inst=self
		
		self.cpu_limit=cpu_limit
		self.wall_limit=cpu_limit*2
		self.problems=problems
		self.engine=engine
		
		self.unsolved=set()
		self.unsolved.update(problems)
		
		self.cpu_adjustation=0.0
	
	
	def wall_time_remaining(self):
		elapsed=get_wall_time()-start_wall_time
		return self.wall_limit-elapsed
	def cpu_time_remaining(self):
		elapsed=get_cpu_time()+self.cpu_adjustation
		return self.cpu_limit-elapsed
	def time_remaining(self):
		return min(self.cpu_time_remaining(), self.wall_time_remaining())

	def problem_time_remaining(self, problem):
		rem=self.time_remaining()
		uns_cnt=len(self.unsolved)
		if self.attempted_cnt==self.solved_cnt:
			cnt=max(uns_cnt/4,min(3, uns_cnt))
			return rem/cnt
		else:
			approx=((1.0*rem*self.attempted_cnt)/uns_cnt-self.solutions_time)/(self.attempted_cnt-self.solved_cnt)
			if uns_cnt==1:
				return min(approx,rem)
			else:
				return min(approx,max(rem-1,1))
	
	def perform_phase(self, problem):
		'''Performs single phase of proof attempts. If last phase was finished, returns False.'''
		if problem.current_phase>=len(self.phases):
			return False
		time_perc,params = self.phases[problem.current_phase]

		problem.current_phase=problem.current_phase+1
		
		forms=selection.select_formulas(problem, params)
		forms_fname=tptp.write_forms_to_file(problem, forms)
		
		#we don't get avaiable time earlier as the previous lines could take a long time
		time=time_perc*self.problem_time_remaining(problem)
		
		if time<=0:
			return False
		
		#print '---remains %d cpu, %d wall, %d problems to solve' % \
		#  	   	(self.cpu_time_remaining(), self.wall_time_remaining(), len(self.unsolved))
		#print '-running phase %d (%s) prover on %s, run time limit is %d s.' % \
		#	 	(problem.current_phase, params, problem, time)


		self.engine.run_into_file(problem.out_fname, problem_file=forms_fname, time=time)
		
		self.handle_finished_prover_run(problem)
			
		return True

	def work(self):
		self.solved_cnt=0
		self.attempted_cnt=0
		self.solutions_time=0
		for problem in self.problems:
			prb_time=self.problem_time_remaining(problem)
			print "%% SZS status Started for %s" % (problem.prob_fname,)
			problem.active=True
			solved=False

			prb_time_init=prb_time_rem=self.problem_time_remaining(problem)
			time_rem=self.time_remaining()
			
			for time_perc,params in self.phases:
				forms=selection.select_formulas(problem, params)
				forms_fname=tptp.write_forms_to_file(problem, forms)
		
				time_zero_point=self.time_remaining()
				time_start_wall=get_wall_time()
		
				self.engine.run_into_file(problem.out_fname, problem_file=forms_fname, time=prb_time_rem*time_perc)
				
				own_elapsed=time_zero_point-self.time_remaining()
				wall_elapsed=get_wall_time()-time_start_wall
				self.cpu_adjustation+=wall_elapsed-own_elapsed
				
				if self.handle_finished_prover_run(problem):
					solved=True
					break
				new_time_rem=self.time_remaining()
				prb_time_rem-=time_rem-new_time_rem;
				if prb_time_rem<=0:
					break
				time_rem=new_time_rem
				problem.current_phase+=1
			if solved:
				print "%% SZS status Theorem for %s" % (problem.prob_fname,)
				self.solved_cnt+=1
				self.solutions_time+=prb_time_init-prb_time_rem
			self.attempted_cnt+=1
			
			problem.active=False
			print "%% SZS status Ended for %s" % (problem.prob_fname,)
		print "Solved %d out of %d." % (self.solved_cnt,len(self.problems))

	def handle_finished_prover_run(self, problem):
		if self.engine.proof_complete(problem.out_fname):
			self.engine.update_proof_axioms(problem.out_fname, problem.axiom_set_set)
			self.unsolved.remove(problem)
			print 'Problem %s solved in phase %s.' % \
			 	   (problem, problem.current_phase)
			return True
		return False
		
	@classmethod
	def get_instance(cls):
		return cls.inst

def work(*args, **kwargs):
	timer=Timer(*args, **kwargs)
	timer.work()	
	
	
	
