#!/usr/bin/python


import os
import signal
import subprocess
import time

class ProcessAlreadyFinishedError(Exception):
	pass

def get_cpu_time():
	times=os.times()
	return reduce(lambda x,y:x+y, times[0:4])

def get_wall_time():
	return os.times()[4]

class Process(object):
	sleep_secs=1

	def __init__(self, cmdl, out_fname=None, handle_subproc=False, **kwargs):
		self.cmdl=cmdl
		self.out_fname=out_fname
		self.po_args=kwargs
		self.po=None
		self.suspended=False
		self.finished=False
		self.forced_finish=False
		self.out_file=None
		self.handle_subproc=handle_subproc
		self._child_pids=None
	
	def __del__(self):
		self.assert_finished()
	
	def start(self):
		args=dict(self.po_args)
		if self.out_fname:
			self.out_file=open(self.out_fname,'w')
			args['stdout']=self.out_file
		self.po=subprocess.Popen(self.cmdl, **args)
	
	def assert_running(self):
		if self.finished:
			raise ProcessAlreadyFinishedError()
		if not self.po:
			self.start()
		if self.suspended:
			self.kill(signal.SIGCONT)
			self.suspended=False
	
	def assert_file_closed(self):
		if self.out_file:
			self.out_file.close()
			self.out_file=None
			
	def assert_finished(self):
		if self.po and not self.finished:
			self.kill(signal.SIGKILL)
			if self.suspended:
				self.kill(signal.SIGCONT)
			self.finished=True
			self.forced_finish=True
		self.assert_file_closed()

	def run_for_some_time(self, secs):
		start_time=get_wall_time()
		self.assert_running()
		while get_wall_time()-start_time<secs:
			time.sleep(self.sleep_secs)
			if self.po.poll()!=None:
				self.finished=True
				self.assert_file_closed()
				break
		else:
			self.kill(signal.SIGSTOP)
			self.suspended=True
	
	def get_child_pids(self):
		if not self._child_pids:
			import processinfo
			ppid=self.po.pid
			children=[]
			pids=processinfo.getPids()
			res=filter(lambda x:processinfo.getPidDetails(x)['ppid']==str(ppid) , pids)
			#empty children lists are not cached as there will probably be children later
			if res:
				self._child_pids=res
			else:
				return []
		return self._child_pids
			
	def kill(self, sig):
		pid=self.po.pid
		os.kill(pid, sig)
		if self.handle_subproc:
			for cpid in self.get_child_pids():
				os.kill(cpid, sig)
				